var scenarioTemplate = {
    bins : [],
    items : []    
}

function readFile(files, importInstructions) {
    if(files != undefined) {
        parseCSV(files[0], importInstructions);
    }
}

function parseCSV(data, importInstructions) {
   if(!data)
       return;
   $('#progress-spinner-loading-files').css('display', 'block');
   Papa.parse(data, {
	complete: function(results) {
        importInstructions(results.data);
        tryEnableLoadButton();
        $('#progress-spinner-loading-files').css('display', 'none');
	}});
}

function importItems(data) {    
    scenarioTemplate.items = [];    
    for(var i = 0; i < data.length; i++) {
        if(data[i].length == 8 && data[i][0] !== "ID")
            scenarioTemplate.items.push(lineToItem(data[i]));
    }
    $('#import-csv-items-btn').removeClass("btn-primary btn-danger btn-success");
    if(scenarioTemplate.items.length == 0) {
        $('#import-csv-items-text').val("Please select a valid file.");
    $('#import-csv-items-btn').addClass("btn-danger");
    } else {
        $('#import-csv-items-btn').addClass("btn-success");
    }	
}

function lineToItem(line) {
    return {
        "id": line[0],          
        "numberOfItems": parseInt( "0" + line[1]),
        "dimensions": {
            "x": parseInt( "0" + line[2]),
            "y": parseInt( "0" + line[3]),
            "z": parseInt( "0" + line[4])
        },
        "weight": parseInt( "0" + line[5]),
        "maximumSurfaceLoads": get_surface_loads((line[6] === "1")),
        "allowedOrientations": get_allowed_orientations((line[7] === "1"))        
        };
}

var get_surface_loads = function(stackable) {
    return stackable ? { } : { x: 0.0, y: 0.0, z: 0.0 };
};

var get_allowed_orientations = function(can_tilt) {
    return can_tilt ? { } : {
        original: true, 
        x: false, y: true, z: false,
        xz: false, yz: false
    };
}

function importBins(data) {    
    scenarioTemplate.bins = [];    
    for(var i = 0; i < data.length; i++) {
        if(data[i].length == 6 && data[i][0] !== "ID")
            scenarioTemplate.bins.push(lineToBin(data[i]));
    }
    $('#import-csv-bins-btn').removeClass("btn-primary btn-danger btn-success");
    if(scenarioTemplate.bins.length == 0) {        
        $('#import-csv-bins-text').val("Please select a valid file.");
        $('#import-csv-bins-btn').addClass("btn-danger");
    } else {
        $('#import-csv-bins-btn').addClass("btn-success");
    }
}

function lineToBin(line) {
    return {
        "id": line[0],          
        "numberOfBins": parseInt( "0" + line[1]),
        "dimensions": {
            "x": parseInt( "0" + line[2]),
            "y": parseInt( "0" + line[3]),
            "z": parseInt( "0" + line[4])
        },
        "maximumWeightCapacity": parseInt(line[5])
        };
}

function resetTemplate() {
    scenarioTemplate.bins = [];
    scenarioTemplate.items = [];
    $('#import-csv-bins').val('');
    $('#import-csv-bins-text').val('No bin csv selected');
    $('#import-csv-items').val('');
    $('#import-csv-items-text').val('No item csv selected');
    $('#load-scenario-btn').prop("disabled", "disabled"); 
    $('#import-csv-items-btn').removeClass("btn-primary btn-danger btn-success");
    $('#import-csv-items-btn').addClass("btn-primary");
    $('#import-csv-bins-btn').removeClass("btn-primary btn-danger btn-success");
    $('#import-csv-bins-btn').addClass("btn-primary");
}

// enable all tooltips
$(function () {   $('[data-toggle="tooltip"]').tooltip() });

// EVENT HANDLER

$('#import-csv-items').change( function() {    
    var input = $(this);
    $('#import-csv-items-text').val(input.val().replace(/\\/g, '/').replace(/.*\//, ''));
    readFile(this.files, importItems);           
});

$('#cancel-load-csv-btn').change(function() {
    resetTemplate();
    $('#loadDialog').modal('hide');
});
    
$('#import-csv-bins').change( function() {
    var input = $(this);
    $('#import-csv-bins-text').val(input.val().replace(/\\/g, '/').replace(/.*\//, '')); 
    readFile(this.files, importBins);       
});

function tryEnableLoadButton() {
    $('#load-scenario-btn').prop("disabled", (!((scenarioTemplate.bins.length > 0) && scenarioTemplate.items.length > 0))); 
}